@@ -1,32 +1,34 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 import string
 import unicodedata
 from collections.abc import Sequence
 from functools import partial
+from pathlib import Path
 from typing import Any, Dict, List, Optional
 from typing import Sequence as SequenceType
 from typing import Tuple, TypeVar, Union
 
 import numpy as np
+from PIL import Image
 
 from doctr.io.image import get_img_shape
-from doctr.utils.geometry import convert_to_relative_coords
+from doctr.utils.geometry import convert_to_relative_coords, extract_crops, extract_rcrops
 
 from .vocabs import VOCABS
 
-__all__ = ['translate', 'encode_string', 'decode_sequence', 'encode_sequences']
+__all__ = ["translate", "encode_string", "decode_sequence", "encode_sequences"]
 
-ImageTensor = TypeVar('ImageTensor')
+ImageTensor = TypeVar("ImageTensor")
 
 
 def translate(
     input_string: str,
     vocab_name: str,
-    unknown_char: str = '■',
+    unknown_char: str = "■",
 ) -> str:
     """Translate a string input in a given vocabulary
 
@@ -41,7 +43,7 @@ def translate(
     if VOCABS.get(vocab_name) is None:
         raise KeyError("output vocabulary must be in vocabs dictionnary")
 
-    translated = ''
+    translated = ""
     for char in input_string:
         if char not in VOCABS[vocab_name]:
             # we need to translate char into a vocab char
@@ -49,8 +51,8 @@ def translate(
                 # remove whitespaces
                 continue
             # normalize character if it is not in vocab
-            char = unicodedata.normalize('NFD', char).encode('ascii', 'ignore').decode('ascii')
-            if char == '' or char not in VOCABS[vocab_name]:
+            char = unicodedata.normalize("NFD", char).encode("ascii", "ignore").decode("ascii")
+            if char == "" or char not in VOCABS[vocab_name]:
                 # if normalization fails or char still not in vocab, return unknown character)
                 char = unknown_char
         translated += char
@@ -70,11 +72,14 @@ def encode_string(
     Returns:
         A list encoding the input_string"""
 
-    return list(map(vocab.index, input_string))  # type: ignore[arg-type]
+    try:
+        return list(map(vocab.index, input_string))
+    except ValueError:
+        raise ValueError("some characters cannot be found in 'vocab'")
 
 
 def decode_sequence(
-    input_seq: Union[np.array, SequenceType[int]],
+    input_seq: Union[np.ndarray, SequenceType[int]],
     mapping: str,
 ) -> str:
     """Given a predefined mapping, decode the sequence of numbers to a string
@@ -92,7 +97,7 @@ def decode_sequence(
     if isinstance(input_seq, np.ndarray) and (input_seq.dtype != np.int_ or input_seq.max() >= len(mapping)):
         raise AssertionError("Input must be an array of int, with max less than mapping size")
 
-    return ''.join(map(mapping.__getitem__, input_seq))
+    return "".join(map(mapping.__getitem__, input_seq))
 
 
 def encode_sequences(
@@ -140,13 +145,13 @@ def encode_sequences(
         default_symbol = pad
     else:  # pad with eos symbol
         default_symbol = eos
-    encoded_data = np.full([len(sequences), target_size], default_symbol, dtype=np.int32)
+    encoded_data: np.ndarray = np.full([len(sequences), target_size], default_symbol, dtype=np.int32)
 
     # Encode the strings
     for idx, seq in enumerate(map(partial(encode_string, vocab=vocab), sequences)):
         if isinstance(pad, int):  # add eos at the end of the sequence
             seq.append(eos)
-        encoded_data[idx, :min(len(seq), target_size)] = seq[:min(len(seq), target_size)]
+        encoded_data[idx, : min(len(seq), target_size)] = seq[: min(len(seq), target_size)]
 
     if isinstance(sos, int):  # place sos symbol at the beginning of each sequence
         if 0 <= sos < len(vocab):
@@ -159,5 +164,22 @@ def encode_sequences(
 
 def convert_target_to_relative(img: ImageTensor, target: Dict[str, Any]) -> Tuple[ImageTensor, Dict[str, Any]]:
 
-    target['boxes'] = convert_to_relative_coords(target['boxes'], get_img_shape(img))
+    target["boxes"] = convert_to_relative_coords(target["boxes"], get_img_shape(img))
     return img, target
+
+
+def crop_bboxes_from_image(img_path: Union[str, Path], geoms: np.ndarray) -> List[np.ndarray]:
+    """Crop a set of bounding boxes from an image
+    Args:
+        img_path: path to the image
+        geoms: a array of polygons of shape (N, 4, 2) or of straight boxes of shape (N, 4)
+    Returns:
+        a list of cropped images
+    """
+    img: np.ndarray = np.array(Image.open(img_path).convert("RGB"))
+    # Polygon
+    if geoms.ndim == 3 and geoms.shape[1:] == (4, 2):
+        return extract_rcrops(img, geoms.astype(dtype=int))
+    if geoms.ndim == 2 and geoms.shape[1] == 4:
+        return extract_crops(img, geoms.astype(dtype=int))
+    raise ValueError("Invalid geometry format")
