@@ -1,19 +1,19 @@
 # Copyright (C) 2021-2022, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 from typing import List, Tuple
 
 import cv2
 import numpy as np
 
-__all__ = ['BarCodeDetector']
+__all__ = ["BarCodeDetector"]
 
 
 class BarCodeDetector:
 
-    """ Implements a Bar-code detector.
+    """Implements a Bar-code detector.
     For now, only horizontal (or with a small angle) bar-codes are supported
 
     Args:
@@ -21,19 +21,15 @@ class BarCodeDetector:
         canny_minval: lower bound for canny hysteresis
         canny_maxval: upper-bound for canny hysteresis
     """
-    def __init__(
-        self,
-        min_size: float = 1 / 6,
-        canny_minval: int = 50,
-        canny_maxval: int = 150
-    ) -> None:
+
+    def __init__(self, min_size: float = 1 / 6, canny_minval: int = 50, canny_maxval: int = 150) -> None:
         self.min_size = min_size
         self.canny_minval = canny_minval
         self.canny_maxval = canny_maxval
 
     def __call__(
         self,
-        img: np.array,
+        img: np.ndarray,
     ) -> List[Tuple[float, float, float, float]]:
         """Detect Barcodes on the image
         Args:
@@ -56,7 +52,7 @@ class BarCodeDetector:
         edges = cv2.dilate(edges, np.ones((1, k), np.uint8))
 
         # Instantiate a barcode-shaped kernel and erode to keep only vertical-bar structures
-        bar_code_kernel = np.zeros((k, 3), np.uint8)
+        bar_code_kernel: np.ndarray = np.zeros((k, 3), np.uint8)
         bar_code_kernel[..., [0, 2]] = 1
         edges = cv2.erode(edges, bar_code_kernel, iterations=1)
 
