@@ -1,10 +1,11 @@
 # Copyright (C) 2021-2022, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 
 import math
+from copy import deepcopy
 from functools import partial
 from typing import Any, Dict, List, Optional, Tuple
 
@@ -16,16 +17,16 @@ from doctr.datasets import VOCABS
 from ...utils.pytorch import load_pretrained_params
 from ..resnet.pytorch import ResNet
 
-__all__ = ['magc_resnet31']
+__all__ = ["magc_resnet31"]
 
 
 default_cfgs: Dict[str, Dict[str, Any]] = {
-    'magc_resnet31': {
-        'mean': (0.694, 0.695, 0.693),
-        'std': (0.299, 0.296, 0.301),
-        'input_shape': (3, 32, 32),
-        'classes': list(VOCABS['french']),
-        'url': 'https://github.com/mindee/doctr/releases/download/v0.4.1/magc_resnet31-857391d8.pt',
+    "magc_resnet31": {
+        "mean": (0.694, 0.695, 0.693),
+        "std": (0.299, 0.296, 0.301),
+        "input_shape": (3, 32, 32),
+        "classes": list(VOCABS["french"]),
+        "url": "https://doctr-static.mindee.com/models?id=v0.4.1/magc_resnet31-857391d8.pt&src=0",
     },
 }
 
@@ -48,6 +49,7 @@ class MAGC(nn.Module):
         headers: int = 8,
         attn_scale: bool = False,
         ratio: float = 0.0625,  # bottleneck ratio of 1/16 as described in paper
+        cfg: Optional[Dict[str, Any]] = None,
     ) -> None:
         super().__init__()
 
@@ -65,7 +67,7 @@ class MAGC(nn.Module):
             nn.Conv2d(self.inplanes, self.planes, kernel_size=1),
             nn.LayerNorm([self.planes, 1, 1]),
             nn.ReLU(inplace=True),
-            nn.Conv2d(self.planes, self.inplanes, kernel_size=1)
+            nn.Conv2d(self.planes, self.inplanes, kernel_size=1),
         )
 
     def forward(self, inputs: torch.Tensor) -> torch.Tensor:
@@ -108,10 +110,17 @@ def _magc_resnet(
     stage_stride: List[int],
     stage_conv: List[bool],
     stage_pooling: List[Optional[Tuple[int, int]]],
+    ignore_keys: Optional[List[str]] = None,
     **kwargs: Any,
 ) -> ResNet:
 
-    kwargs['num_classes'] = kwargs.get('num_classes', len(default_cfgs[arch]['classes']))
+    kwargs["num_classes"] = kwargs.get("num_classes", len(default_cfgs[arch]["classes"]))
+    kwargs["classes"] = kwargs.get("classes", default_cfgs[arch]["classes"])
+
+    _cfg = deepcopy(default_cfgs[arch])
+    _cfg["num_classes"] = kwargs["num_classes"]
+    _cfg["classes"] = kwargs["classes"]
+    kwargs.pop("classes")
 
     # Build the model
     model = ResNet(
@@ -121,11 +130,15 @@ def _magc_resnet(
         stage_conv,
         stage_pooling,
         attn_module=partial(MAGC, headers=8, attn_scale=True),
+        cfg=_cfg,
         **kwargs,
     )
     # Load pretrained parameters
     if pretrained:
-        load_pretrained_params(model, default_cfgs[arch]['url'])
+        # The number of classes is not the same as the number of classes in the pretrained model =>
+        # remove the last layer weights
+        _ignore_keys = ignore_keys if kwargs["num_classes"] != len(default_cfgs[arch]["classes"]) else None
+        load_pretrained_params(model, default_cfgs[arch]["url"], ignore_keys=_ignore_keys)
 
     return model
 
@@ -149,7 +162,7 @@ def magc_resnet31(pretrained: bool = False, **kwargs: Any) -> ResNet:
     """
 
     return _magc_resnet(
-        'magc_resnet31',
+        "magc_resnet31",
         pretrained,
         [1, 2, 5, 3],
         [256, 256, 512, 512],
@@ -158,5 +171,6 @@ def magc_resnet31(pretrained: bool = False, **kwargs: Any) -> ResNet:
         [(2, 2), (2, 1), None, None],
         origin_stem=False,
         stem_channels=128,
+        ignore_keys=["13.weight", "13.bias"],
         **kwargs,
     )
