@@ -1,13 +1,14 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 
-from typing import Dict, Optional, Any
+from typing import Any, Dict, Optional
+
 from doctr.utils.repr import NestedObject
 
-__all__ = ['BaseModel']
+__all__ = ["BaseModel"]
 
 
 class BaseModel(NestedObject):
