@@ -1,7 +1,7 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
-# This program is licensed under the Apache License version 2.
-# See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
+# This program is licensed under the Apache License 2.0.
+# See LICENSE or go to <https://opensource.org/licenses/Apache-2.0> for full license details.
 
 from typing import Dict, List, Optional, Tuple
 
@@ -10,8 +10,17 @@ import numpy as np
 from scipy.optimize import linear_sum_assignment
 from unidecode import unidecode
 
-__all__ = ['TextMatch', 'box_iou', 'box_ioa', 'mask_iou', 'polygon_iou',
-           'nms', 'LocalizationConfusion', 'OCRMetric', 'DetectionMetric']
+__all__ = [
+    "TextMatch",
+    "box_iou",
+    "box_ioa",
+    "mask_iou",
+    "polygon_iou",
+    "nms",
+    "LocalizationConfusion",
+    "OCRMetric",
+    "DetectionMetric",
+]
 
 
 def string_match(word1: str, word2: str) -> Tuple[bool, bool, bool, bool]:
@@ -25,12 +34,12 @@ def string_match(word1: str, word2: str) -> Tuple[bool, bool, bool, bool]:
         a tuple with booleans specifying respectively whether the raw strings, their lower-case counterparts, their
             unidecode counterparts and their lower-case unidecode counterparts match
     """
-    raw_match = (word1 == word2)
-    caseless_match = (word1.lower() == word2.lower())
-    unidecode_match = (unidecode(word1) == unidecode(word2))
+    raw_match = word1 == word2
+    caseless_match = word1.lower() == word2.lower()
+    unidecode_match = unidecode(word1) == unidecode(word2)
 
     # Warning: the order is important here otherwise the pair ("EUR", "€") cannot be matched
-    unicase_match = (unidecode(word1).lower() == unidecode(word2).lower())
+    unicase_match = unidecode(word1).lower() == unidecode(word2).lower()
 
     return raw_match, caseless_match, unidecode_match, unicase_match
 
@@ -58,11 +67,10 @@ class TextMatch:
     where :math:`\mathcal{W}` is the set of all possible character sequences,
     :math:`N` is a strictly positive integer.
 
-    Example::
-        >>> from doctr.utils import TextMatch
-        >>> metric = TextMatch()
-        >>> metric.update(['Hello', 'world'], ['hello', 'world'])
-        >>> metric.summary()
+    >>> from doctr.utils import TextMatch
+    >>> metric = TextMatch()
+    >>> metric.update(['Hello', 'world'], ['hello', 'world'])
+    >>> metric.summary()
     """
 
     def __init__(self) -> None:
@@ -123,11 +131,12 @@ def box_iou(boxes_1: np.ndarray, boxes_2: np.ndarray) -> np.ndarray:
     Args:
         boxes_1: bounding boxes of shape (N, 4) in format (xmin, ymin, xmax, ymax)
         boxes_2: bounding boxes of shape (M, 4) in format (xmin, ymin, xmax, ymax)
+
     Returns:
         the IoU matrix of shape (N, M)
     """
 
-    iou_mat = np.zeros((boxes_1.shape[0], boxes_2.shape[0]), dtype=np.float32)
+    iou_mat: np.ndarray = np.zeros((boxes_1.shape[0], boxes_2.shape[0]), dtype=np.float32)
 
     if boxes_1.shape[0] > 0 and boxes_2.shape[0] > 0:
         l1, t1, r1, b1 = np.split(boxes_1, 4, axis=1)
@@ -152,11 +161,12 @@ def box_ioa(boxes_1: np.ndarray, boxes_2: np.ndarray) -> np.ndarray:
     Args:
         boxes_1: bounding boxes of shape (N, 4) in format (xmin, ymin, xmax, ymax)
         boxes_2: bounding boxes of shape (M, 4) in format (xmin, ymin, xmax, ymax)
+
     Returns:
         the IoA matrix of shape (N, M)
     """
 
-    ioa_mat = np.zeros((boxes_1.shape[0], boxes_2.shape[0]), dtype=np.float32)
+    ioa_mat: np.ndarray = np.zeros((boxes_1.shape[0], boxes_2.shape[0]), dtype=np.float32)
 
     if boxes_1.shape[0] > 0 and boxes_2.shape[0] > 0:
         l1, t1, r1, b1 = np.split(boxes_1, 4, axis=1)
@@ -188,7 +198,7 @@ def mask_iou(masks_1: np.ndarray, masks_2: np.ndarray) -> np.ndarray:
     if masks_1.shape[1:] != masks_2.shape[1:]:
         raise AssertionError("both boolean masks should have the same spatial shape")
 
-    iou_mat = np.zeros((masks_1.shape[0], masks_2.shape[0]), dtype=np.float32)
+    iou_mat: np.ndarray = np.zeros((masks_1.shape[0], masks_2.shape[0]), dtype=np.float32)
 
     if masks_1.shape[0] > 0 and masks_2.shape[0] > 0:
         axes = tuple(range(2, masks_1.ndim + 1))
@@ -200,10 +210,7 @@ def mask_iou(masks_1: np.ndarray, masks_2: np.ndarray) -> np.ndarray:
 
 
 def polygon_iou(
-    polys_1: np.ndarray,
-    polys_2: np.ndarray,
-    mask_shape: Tuple[int, int],
-    use_broadcasting: bool = False
+    polys_1: np.ndarray, polys_2: np.ndarray, mask_shape: Tuple[int, int], use_broadcasting: bool = False
 ) -> np.ndarray:
     """Computes the IoU between two sets of rotated bounding boxes
 
@@ -220,7 +227,7 @@ def polygon_iou(
     if polys_1.ndim != 3 or polys_2.ndim != 3:
         raise AssertionError("expects boxes to be in format (N, 4, 2)")
 
-    iou_mat = np.zeros((polys_1.shape[0], polys_2.shape[0]), dtype=np.float32)
+    iou_mat: np.ndarray = np.zeros((polys_1.shape[0], polys_2.shape[0]), dtype=np.float32)
 
     if polys_1.shape[0] > 0 and polys_2.shape[0] > 0:
         if use_broadcasting:
@@ -249,7 +256,7 @@ def _rbox_to_mask(box: np.ndarray, shape: Tuple[int, int]) -> np.ndarray:
         the boolean mask of the specified shape
     """
 
-    mask = np.zeros(shape, dtype=np.uint8)
+    mask: np.ndarray = np.zeros(shape, dtype=np.uint8)
     # Get absolute coords
     if box.dtype != int:
         abs_box = box.copy()
@@ -275,15 +282,15 @@ def rbox_to_mask(boxes: np.ndarray, shape: Tuple[int, int]) -> np.ndarray:
         the boolean masks of shape (N, H, W)
     """
 
-    masks = np.zeros((boxes.shape[0], *shape), dtype=np.uint8)
+    masks: np.ndarray = np.zeros((boxes.shape[0], *shape), dtype=np.uint8)
 
     if boxes.shape[0] > 0:
         # Get absolute coordinates
-        if boxes.dtype != np.int:
+        if boxes.dtype != int:
             abs_boxes = boxes.copy()
             abs_boxes[:, :, 0] = abs_boxes[:, :, 0] * shape[1]
             abs_boxes[:, :, 1] = abs_boxes[:, :, 1] * shape[0]
-            abs_boxes = abs_boxes.round().astype(np.int)
+            abs_boxes = abs_boxes.round().astype(int)
         else:
             abs_boxes = boxes
             abs_boxes[:, 2:] = abs_boxes[:, 2:] + 1
@@ -294,7 +301,7 @@ def rbox_to_mask(boxes: np.ndarray, shape: Tuple[int, int]) -> np.ndarray:
     return masks.astype(bool)
 
 
-def nms(boxes: np.ndarray, thresh: float = .5) -> List[int]:
+def nms(boxes: np.ndarray, thresh: float = 0.5) -> List[int]:
     """Perform non-max suppression, borrowed from <https://github.com/rbgirshick/fast-rcnn>`_.
 
     Args:
@@ -358,12 +365,11 @@ class LocalizationConfusion:
     where :math:`\mathcal{B}` is the set of possible bounding boxes,
     :math:`N` (number of ground truths) and :math:`M` (number of predictions) are strictly positive integers.
 
-    Example::
-        >>> import numpy as np
-        >>> from doctr.utils import LocalizationConfusion
-        >>> metric = LocalizationConfusion(iou_thresh=0.5)
-        >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]))
-        >>> metric.summary()
+    >>> import numpy as np
+    >>> from doctr.utils import LocalizationConfusion
+    >>> metric = LocalizationConfusion(iou_thresh=0.5)
+    >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]))
+    >>> metric.summary()
 
     Args:
         iou_thresh: minimum IoU to consider a pair of prediction and ground truth as a match
@@ -431,7 +437,7 @@ class LocalizationConfusion:
         self.num_gts = 0
         self.num_preds = 0
         self.matches = 0
-        self.tot_iou = 0.
+        self.tot_iou = 0.0
 
 
 class OCRMetric:
@@ -463,13 +469,12 @@ class OCRMetric:
     :math:`\mathcal{L}` is the set of possible character sequences,
     :math:`N` (number of ground truths) and :math:`M` (number of predictions) are strictly positive integers.
 
-    Example::
-        >>> import numpy as np
-        >>> from doctr.utils import OCRMetric
-        >>> metric = OCRMetric(iou_thresh=0.5)
-        >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]),
-        ['hello'], ['hello', 'world'])
-        >>> metric.summary()
+    >>> import numpy as np
+    >>> from doctr.utils import OCRMetric
+    >>> metric = OCRMetric(iou_thresh=0.5)
+    >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]),
+    >>>               ['hello'], ['hello', 'world'])
+    >>> metric.summary()
 
     Args:
         iou_thresh: minimum IoU to consider a pair of prediction and ground truth as a match
@@ -508,8 +513,9 @@ class OCRMetric:
         """
 
         if gt_boxes.shape[0] != len(gt_labels) or pred_boxes.shape[0] != len(pred_labels):
-            raise AssertionError("there should be the same number of boxes and string both for the ground truth "
-                                 "and the predictions")
+            raise AssertionError(
+                "there should be the same number of boxes and string both for the ground truth " "and the predictions"
+            )
 
         # Compute IoU
         if pred_boxes.shape[0] > 0:
@@ -565,7 +571,7 @@ class OCRMetric:
     def reset(self) -> None:
         self.num_gts = 0
         self.num_preds = 0
-        self.tot_iou = 0.
+        self.tot_iou = 0.0
         self.raw_matches = 0
         self.caseless_matches = 0
         self.unidecode_matches = 0
@@ -601,13 +607,12 @@ class DetectionMetric:
     :math:`\mathcal{C}` is the set of possible class indices,
     :math:`N` (number of ground truths) and :math:`M` (number of predictions) are strictly positive integers.
 
-    Example::
-        >>> import numpy as np
-        >>> from doctr.utils import DetectionMetric
-        >>> metric = DetectionMetric(iou_thresh=0.5)
-        >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]),
-        np.zeros(1, dtype=np.int64), np.array([0, 1], dtype=np.int64))
-        >>> metric.summary()
+    >>> import numpy as np
+    >>> from doctr.utils import DetectionMetric
+    >>> metric = DetectionMetric(iou_thresh=0.5)
+    >>> metric.update(np.asarray([[0, 0, 100, 100]]), np.asarray([[0, 0, 70, 70], [110, 95, 200, 150]]),
+    >>>               np.zeros(1, dtype=np.int64), np.array([0, 1], dtype=np.int64))
+    >>> metric.summary()
 
     Args:
         iou_thresh: minimum IoU to consider a pair of prediction and ground truth as a match
@@ -646,8 +651,9 @@ class DetectionMetric:
         """
 
         if gt_boxes.shape[0] != gt_labels.shape[0] or pred_boxes.shape[0] != pred_labels.shape[0]:
-            raise AssertionError("there should be the same number of boxes and string both for the ground truth "
-                                 "and the predictions")
+            raise AssertionError(
+                "there should be the same number of boxes and string both for the ground truth " "and the predictions"
+            )
 
         # Compute IoU
         if pred_boxes.shape[0] > 0:
@@ -688,5 +694,5 @@ class DetectionMetric:
     def reset(self) -> None:
         self.num_gts = 0
         self.num_preds = 0
-        self.tot_iou = 0.
+        self.tot_iou = 0.0
         self.num_matches = 0
